<?php
/**
 * Pocket Project
 * 
 * @copyright 2006 Power-Station co., Ltd, ALL Rights Reserved,
 */

/**
 * INCLUDE PEAR DB
 */
require_once(PEAR_DIR . 'DB.php');

/**
 *  DBクラス
 *
 *  @author     Shogo Minami
 *  @access     public
 *  @package    Pocket
 */
class Pocket_DB
{
    /** @var    object  DB_${type}          DB接続オブジェクト */
    var $db;

    /** @var    array   データソース属性配列 */
    var $dsn;

    /**
     *  コンストラクタ
     *
     *  @access public
     *  @param  array   $dsn                データソース属性配列
     */
    function Pocket_DB($dsn)
    {
        $this->dsn = $dsn;
        $this->db = DB::connect($this->dsn);
        // エラーチェック
        if (DB::isError($this->db))
        {
            pocket_dispError(array('データベースへの接続に失敗しました。'));
            exit;
        }

        $this->db->setFetchMode(DB_FETCHMODE_ASSOC);
        $this->db->query('SET NAMES utf8');

        // シャットダウン時に実行する関数を登録
        register_shutdown_function("destroyDBConnect");
    }

    /**
     *  インスタンス取得
     *
     *  @access public
     */
    function &getInstance()
    {
        static $_singleton;
        if ($_singleton === null)
        {
            $_singleton = new Pocket_DB($GLOBALS['_POCKET_DSN']);
        }
        return $_singleton;
    }

    /**
     *  DB取得
     *
     *  @access public
     */
    function &getDB()
    {
        return $this->db;
    }

    /**
     *  DSN取得
     *
     *  @access public
     */
    function getDSN()
    {
        return $this->dsn;
    }

    /**
     *  最初の行の最初のカラムを取得
     *
     *  @access public
     *  @param  string  $sql                SQL文字列
     *  @param  array   $params             パラメータ配列
     */
    function getOne($sql, $params = array())
    {
        $sql = $this->db->modifyLimitQuery($sql, 0, 1, $params);
        $result =& $this->db->getOne($sql, $params);
        if (DB::isError($result))
        {
            return null;
        }
        return $result;
    }

    /**
     *  最初のカラムを取得
     *
     *  @access public
     *  @param  string  $sql                SQL文字列
     *  @param  array   $params             パラメータ配列
     */
    function getCol($sql, $params = array())
    {
        $result =& $this->db->getCol($sql, 0, $params);
        if (DB::isError($result))
        {
            return array();
        }
        return $result;
    }

    /**
     *  最初のカラムを取得（Limit）
     *
     *  @access public
     *  @param  string  $sql                SQL文字列
     *  @param  integer $from               取得開始行
     *  @param  integer $count              取得行数
     *  @param  array   $params             パラメータ配列
     */
    function getColLimit($sql, $from, $count, $params = array())
    {
        $sql = $this->db->modifyLimitQuery($sql, intval($from), intval($count), $params);
        return $this->getCol($sql, $params);
    }

    /**
     *  最初のカラムを取得（Page）
     *
     *  @access public
     *  @param  string  $sql                SQL文字列
     *  @param  integer $page               取得ページ
     *  @param  integer $count              １ページの行数
     *  @param  array   $params             パラメータ配列
     */
    function getColPage($sql, $page, $count, $params = array())
    {
        $from = (intval($page) - 1) * intval($count);
        return $this->getColLimit($sql, $from, $count, $params);
    }

    /**
     *  連想配列を取得
     *
     *  @access public
     *  @param  string  $sql                SQL文字列
     *  @param  array   $params             パラメータ配列
     */
    function getAssoc($sql, $params = array())
    {
        $result =& $this->db->getAssoc($sql, false, $params);
        if (DB::isError($result))
        {
            return array();
        }
        return $result;
    }

    /**
     *  連想配列を取得（Limit）
     *
     *  @access public
     *  @param  string  $sql                SQL文字列
     *  @param  integer $from               取得開始行
     *  @param  integer $count              取得行数
     *  @param  array   $params             パラメータ配列
     */
    function getAssocLimit($sql, $from, $count, $params = array())
    {
        $sql = $this->db->modifyLimitQuery($sql, intval($from), intval($count), $params);
        return $this->getAssoc($sql, $params);
    }

    /**
     *  最初の行を取得
     *
     *  @access public
     *  @param  string  $sql                SQL文字列
     *  @param  array   $params             パラメータ配列
     */
    function getRow($sql, $params = array())
    {
        $sql = $this->db->modifyLimitQuery($sql, 0, 1, $params);
        $result =& $this->db->getRow($sql, $params);
        if (DB::isError($result))
        {
            return array();
        }
        return $result;
    }

    /**
     *  全てのデータを取得
     *
     *  @access public
     *  @param  string  $sql                SQL文字列
     *  @param  array   $params             パラメータ配列
     */
    function getAll($sql, $params = array())
    {
        $result =& $this->db->getAll($sql, $params);
        if (DB::isError($result))
        {
            return array();
        }
        return $result;
    }

    /**
     *  全てのデータを取得（Limit）
     *
     *  @access public
     *  @param  string  $sql                SQL文字列
     *  @param  integer $from               取得開始行
     *  @param  integer $count              取得行数
     *  @param  array   $params             パラメータ配列
     */
    function getAllLimit($sql, $from, $count, $params = array())
    {
        $sql = $this->db->modifyLimitQuery($sql, intval($from), intval($count), $params);
        return $this->getAll($sql, $params);
    }

    /**
     *  全てのデータを取得（Page）
     *
     *  @access public
     *  @param  string  $sql                SQL文字列
     *  @param  integer $page               取得ページ
     *  @param  integer $count              １ページの行数
     *  @param  array   $params             パラメータ配列
     */
    function getAllPage($sql, $page, $count, $params = array())
    {
        $from = (intval($page) - 1) * intval($count);
        return $this->getAllLimit($sql, $from, $count, $params);
    }

    /**
     *  ランダム関数文字列を取得
     *
     *  @access public
     */
    function getRandomFunction()
    {
        if ($this->db->phptype == 'mysql')
        {
            return 'Rand()';
        }
        elseif ($this->db->phptype == 'pgsql')
        {
            return 'Random()';
        }
        else {
            return '';
        }
    }

    /**
     *  最後に自動生成された値を取得
     *
     *  @access public
     *  @param  string  $seq_name           シーケンス名称（MySQLの場合は未使用）
     */
    function insertId($seq_name = '')
    {
        if ($this->db->phptype == 'mysql')
        {
            return $this->db->getOne('SELECT LAST_INSERT_ID()');
        }
        elseif ($this->db->phptype == 'pgsql')
        {
            return $this->db->getOne("SELECT CURRVAL('" . $seq_name . "')");
            /* v8.1以降 */
            //return $this->db->getOne('SELECT LASTVAL()');
        }
        else {
            return null;
        }
    }

    /**
     *  データ更新クエリによって変更された行数を取得
     *
     *  @access public
     */
    function affectedRows()
    {
        return $this->db->affectedRows();
    }

    /**
     *  クエリを実行
     *
     *  @access public
     *  @param  string  $sql                SQL文字列
     *  @param  array   $params             パラメータ配列
     */
    function &query($sql, $params = array())
    {
        return $this->db->query($sql, $params);
    }

    /**
     *  データを登録
     *
     *  @access public
     *  @param  string  $table              テーブル名称
     *  @param  array   $fields             フィールド値配列
     *  @param  string  $seq_name           シーケンス名称（MySQLの場合は未使用）
     */
    function insert($table, $fields, $seq_name = '')
    {
        $result = $this->db->autoExecute($table, $fields, DB_AUTOQUERY_INSERT);
        if (DB::isError($result))
        {
            return false;
        }
        return $this->insertId($seq_name);
    }

    /**
     *  データを更新
     *
     *  @access public
     *  @param  string  $table              テーブル名称
     *  @param  array   $fields             フィールド値配列
     *  @param  array   $where              条件値配列
     */
    function update($table, $fields, $where)
    {
        $where = $this->_makeWhere($where);
        $result = $this->db->autoExecute($table, $fields, DB_AUTOQUERY_UPDATE, $where);
        if (DB::isError($result))
        {
            return false;
        }
        return true;
    }

    /**
     *  データを削除
     *
     *  @access public
     *  @param  string  $table              テーブル名称
     *  @param  string  $key                キー名称
     *  @param  integer $value              値
     */
    function delete($table, $key, $value)
    {
        if (!isset($key) || !isset($value)) {return false;}
        $sql = 'DELETE FROM ' . $table . ' WHERE ' . $key . ' = ?';
        $params = array($value);
        return $this->query($sql, $params);
    }

    /**
     *  リテラルとして安全に使用できるように入力内容を整形
     *
     *  @access public
     *  @param  string  $value              文字列
     */
    function _quoteSmart($value)
    {
        return $this->db->quoteSmart($value);
    }

    /**
     *  条件文を作成
     *
     *  @access public
     *  @param  array   $where              条件値配列
     */
    function _makeWhere($where)
    {
        if (!is_array($where))
        {
            return $where;
        }

        $_where = '';
        $first = true;
        foreach ($where as $key => $value)
        {
            if ($first)
            {
                $first = false;
            }
            else {
                $_where .= ' AND ';
            }
            $_where .= $key . ' = ' . $this->_quoteSmart($value);
        }
        return $_where;
    }
}

/**
 *  データベースとの接続を破棄
 *
 *  @access public
 */
function destroyDBConnect()
{
    $pdb =& Pocket_DB::getInstance();
    if ($pdb && $pdb->db)
    {
        $pdb->db->disconnect();
    }
}

?>
